-- Lulu88520
-- Script pour le pack de plateau ROBUST
-- @date  19/12/16
-- https://www.facebook.com/paulo5090r


ROBUST = {};

function ROBUST.prerequisitesPresent(specializations)
	  return true;
end;

function ROBUST:load(savegame)
	 self.toggleSupport = SpecializationUtil.callSpecializationsFunction("toggleSupport");  
    self.manualSupportAnimation = getXMLString(self.xmlFile, "vehicle.manualSupport#animationName");
    self.supportRangeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.manualSupport#rangeIndex"));
    self.isInRangeSupport = false;
    self.isSupportDown = true;	
end;

function ROBUST:delete()
end;

function ROBUST:readStream(streamId, connection)
self:toggleSupport(streamReadBool(streamId), true);		
end;

function ROBUST:writeStream(streamId, connection)
streamWriteBool(streamId, self.isSupportDown);
end;

function ROBUST:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ROBUST:keyEvent(unicode, sym, modifier, isDown) 
end;

function ROBUST:update(dt)
         if self:getIsActive() then		
		if self.isClient and self:getIsActiveForInput(false) and not self:hasInputConflictWithSelection() then
			-- Show warning about the support being down
			if self.isSupportDown then				
				g_currentMission:showBlinkingWarning(g_i18n:getText("BJR_SUPPORT_WARNING"), 7000);
			end;
		end;
	end;
         if self.supportRangeIndex then
		-- Manage key events for inrange support --
		if self.isInRangeSupport then
			if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
				if self.isSupportDown then
					self:toggleSupport(false);
				else
					self:toggleSupport(true);
				end;
			end;
			-- Display key when in range --
			if self.isSupportDown then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("BJR_RAISE_SUPPORT")), InputBinding.LOWER_IMPLEMENT);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("BJR_LOWER_SUPPORT")), InputBinding.LOWER_IMPLEMENT);
			end;
		end;
	end;
end;

function ROBUST:updateTick(dt)

 if self.supportRangeIndex then
		if g_currentMission.player ~= nil then
			-- Getting the distance between the player and the support
			local nearestDistance = 2.5; --max distance allowed
			local px, py, pz = getWorldTranslation(self.supportRangeIndex);
			local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
			local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
			if distance < nearestDistance then
				self.isInRangeSupport = true;
			else
				self.isInRangeSupport = false;
			end;
		end;
	else
		self.isInRangeSupport = false;
	end;
end;

function ROBUST:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles then
	local isSupportDown = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. "#isSupportDown"), false);
	self:toggleSupport(isSupportDown);	
	end; 
end;

function ROBUST:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'isSupportDown="' .. tostring(self.isSupportDown) .. '"';
    return attributes, nil;
end;
  
function ROBUST:onDetach()
end;

function ROBUST:onAttach(attacherVehicle)
end;

function ROBUST:draw() 
end;

function ROBUST:toggleSupport(isSupportDown, noEventSend)
	ToggleSupportEvent.sendEvent(self, isSupportDown, noEventSend);	
	
	if not isSupportDown then
		if self.manualSupportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.manualSupportAnimation, -1, nil, true);
		end;
	else
		if self.manualSupportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.manualSupportAnimation, 1, nil, true);
		end;
	end;
	
	self.isSupportDown = isSupportDown;
end;
-- Support event

ToggleSupportEvent = {};
ToggleSupportEvent_mt = Class(ToggleSupportEvent, Event);

InitEventClass(ToggleSupportEvent, "ToggleSupportEvent");

function ToggleSupportEvent:emptyNew()
    local self = Event:new(ToggleSupportEvent_mt);
	self.className="ToggleSupportEvent";
    return self;
end;

function ToggleSupportEvent:new(vehicle, isSupportDown)
    local self = ToggleSupportEvent:emptyNew()
    self.vehicle = vehicle;
	self.isSupportDown = isSupportDown;
    return self;
end;

function ToggleSupportEvent:readStream(streamId, connection) 
	local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);
	
	self.isSupportDown = streamReadBool(streamId);		
	self.vehicle:toggleSupport(self.isSupportDown, true);
	
	if not connection:getIsServer() then -- if server read then send to clients
		g_server:broadcastEvent(ToggleSupportEvent:new(self.vehicle, self.isSupportDown), nil, connection, self.vehicle);
	end;	
end;
if g_currentModName ~= "FS17_Robust_Trailer_Pack" then
	while g_currentModName ~= "FS17_Robust_Trailer_Pack" do
	print("Error: The mod must not be renamed. It must be called FS17_Robust_Trailer_Pack.zip");	
		end;
end;
function ToggleSupportEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.isSupportDown);
end;
function ToggleSupportEvent.sendEvent(vehicle, isSupportDown, noEventSend)	
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ToggleSupportEvent:new(vehicle, isSupportDown), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ToggleSupportEvent:new(vehicle, isSupportDown));
		end;
	end;
end;